<?php
/*========================================================================*\
|| ###################################################################### ||
|| # vBulletin 6.0.3 - Licence Number LN98610432
|| # ------------------------------------------------------------------ # ||
|| # Copyright 2000-2024 MH Sub I, LLC dba vBulletin. All Rights Reserved.  # ||
|| # This file may not be redistributed in whole or significant part.   # ||
|| # ----------------- VBULLETIN IS NOT FREE SOFTWARE ----------------- # ||
|| # http://www.vbulletin.com | http://www.vbulletin.com/license.html   # ||
|| ###################################################################### ||
\*========================================================================*/

class vB_Upgrade_600a1 extends vB_Upgrade_Version
{
	public $PREV_VERSION = '5.7.4+';
	public $VERSION_COMPAT_STARTS = '5.7.4';
	public $VERSION_COMPAT_ENDS   = '5.7.99';

	// Update some meta data for refactored nodevote datastore struture
	public function step_1()
	{
		$this->show_message($this->phrase['version'][$this->SHORT_VERSION]['rebuilding_nodevotes']);
		// Changed how we do the string-keying so that any character is allowed for the votetype labels.
		vB_Library::instance('nodevote')->reloadNodevoteMetaDataFromDB();
	}

	// Some of the reaction save logic now requires this new table to exist first.
	public function step_2()
	{
		$this->run_query(
			sprintf($this->phrase['vbphrase']['create_table'], TABLE_PREFIX . 'reactionoption'),
			"
				CREATE TABLE `" . TABLE_PREFIX . "reactionoption` (
					`votetypeid` SMALLINT UNSIGNED NOT NULL,
					`enabled` TINYINT SIGNED NOT NULL DEFAULT '0',
					`user_rep_factor` TINYINT SIGNED NOT NULL DEFAULT '0',
					`user_like_countable` TINYINT SIGNED NOT NULL DEFAULT '0',
					`emojihtml` MEDIUMTEXT NOT NULL,
					`system` TINYINT SIGNED NOT NULL DEFAULT '0',
					`guid` VARCHAR(255) NULL DEFAULT NULL,
					`order` INT SIGNED NOT NULL DEFAULT '0',
					`filedataid` INT UNSIGNED NOT NULL DEFAULT '0',
					UNIQUE INDEX `votetypeid` (`votetypeid`),
					INDEX `filedataid` (`filedataid`)
				) ENGINE = " . $this->hightrafficengine . "
			",
			self::MYSQL_ERROR_TABLE_EXISTS
		);
	}

	// Add default reaction nodevote data
	public function step_3()
	{
		$this->show_message($this->phrase['version'][$this->SHORT_VERSION]['inserting_topic_reactions']);
		$this->addDefaultReactionsGroup();
		// Updated addDefaultReactions() will add AND enable the new emojis by default.
		$this->addDefaultReactions();
	}

	// Porting reputation to reactions: Insert "thumbs up" reactions for any old "likes"
	public function step_4($data = [])
	{
		$thumbsupReaction = $this->getThumbsUpNodevoteData();
		// If something went wrong with step_2() and we don't have the thumbs up reaction inserted, we cannot continue.
		// Skip message for now. May want a different message here?
		if (empty($thumbsupReaction))
		{
			$this->skip_message();
			return;
		}

		if (empty($data['startat']))
		{
			// We're starting with more than just the "thumbs up" as "reputable". In order to make this step re-entrant
			// AFTER we've upgraded to vb6, we cannot run this more than once, because then we might accidentally
			// count a vb6-added reputation record (e.g. from a "grinning face" reaction) as "thumbs up".
			if ($this->iRan(__FUNCTION__))
			{
				return;
			}

			$this->show_message(sprintf($this->phrase['vbphrase']['update_table_x'], TABLE_PREFIX . 'nodevote', 1, 2));
		}

		/*
		$thumbsupReaction:
    [label] => thumbs up
    [votetypeid] => 11418
    [votegroupid] => 45
		 */
		// Copy like counts over
		$queryParams = [
			'votetypeid' => $thumbsupReaction['votetypeid'],
			'votegroupid' => $thumbsupReaction['votegroupid'],
		];
		$walker = new vB_UpdateTableWalker(vB::getDBAssertor());
		$walker->setBatchSize($this->getBatchSize('xlarge', __FUNCTION__));
		$walker->setMaxQuery('vBInstall:getMaxReputationid');
		$walker->setNextidQuery('vBForum:reputation', 'reputationid');
		$walker->setCallbackQuery('vBInstall:copyReputationToNodevotes', $queryParams);

		return $this->updateByWalker($walker, $data);
	}

	// Porting reputation to reactions: fix aggregate counts after nodevotes were potentially changed in step_3
	public function step_5($data = [])
	{
		$thumbsupReaction = $this->getThumbsUpNodevoteData();
		// If something went wrong with step_2() and we don't have the thumbs up reaction inserted, we cannot continue.
		// Skip message for now. May want a different message here?
		if (empty($thumbsupReaction))
		{
			$this->skip_message();
			return;
		}

		if (empty($data['startat']))
		{
			$this->show_message(sprintf($this->phrase['vbphrase']['update_table_x'], TABLE_PREFIX . 'nodevote', 2, 2));
		}

		$walker = new vB_UpdateTableWalker(vB::getDBAssertor());
		$walker->setBatchSize($this->getBatchSize('xlarge', __FUNCTION__));
		$walker->setMaxQuery('vBInstall:getMaxNodevoteNodeid');
		$walker->setNextidQuery('nodevote', 'nodeid');
		$walker->setCallbackQuery('fixNodevoteAggregate');

		return $this->updateByWalker($walker, $data);
	}
}

class vB_Upgrade_600a3 extends vB_Upgrade_Version
{
	public function step_1()
	{
		$this->run_query(
			sprintf($this->phrase['vbphrase']['create_table'], TABLE_PREFIX . 'systemevent'),
			"
				CREATE TABLE " . TABLE_PREFIX . "systemevent (
					systemeventid INT NOT NULL AUTO_INCREMENT,
					runafter INT NOT NULL,
					priority INT NOT NULL,
					data BLOB NOT NULL,
					processkey VARBINARY(50) NULL,
					processtime INT NOT NULL,
					status VARCHAR(50) NOT NULL,
					errormessage TEXT NOT NULL,
					KEY processkey(processkey),
					KEY status (status, runafter),
					PRIMARY KEY (systemeventid)
				) ENGINE = " . $this->hightrafficengine . "
			",
			self::MYSQL_ERROR_TABLE_EXISTS
		);
	}
}

class vB_Upgrade_600a7 extends vB_Upgrade_Version
{
	public function step_1($data = [])
	{
		return $this->file_load_legacy_style($data);
	}

	public function step_2()
	{
		$this->show_message(sprintf($this->phrase['vbphrase']['update_table_x'], TABLE_PREFIX . 'style', 1, 2));

		$db = vB::getDBAssertor();

		//We could do this an one custom query including the looped updates below but it's not worth the trouble
		//There just aren't going to be that many styles that are direct children of the root.
		$legacy = $db->getRow('style', [
			vB_dB_Query::COLUMNS_KEY => ['styleid'],
			'guid' => 'vbulletin-theme-legacy-668e390263dc4ebd89f967b785bc47da',
		]);

		//these styles should have been created in step 1, but if not ... not much we can do about it.
		if ($legacy)
		{
			$legacywritable = $db->getRow('style', [
				vB_dB_Query::COLUMNS_KEY => ['styleid', 'parentlist'],
				vB_dB_Query::CONDITIONS_KEY => [
					'parentid' => $legacy['styleid'],
					['field' => 'guid', 'value' => '-writable-', vB_dB_Query::OPERATOR_KEY => vB_dB_Query::OPERATOR_INCLUDES],
				]
			]);

			$stylelib = vB_Library::instance('style');
			if ($legacywritable)
			{
				//Assume any styles that inherit directly from the master style and *aren't* themes are vB5 styles and
				//move them to the vB5 style shim.
				$styles = $db->getColumn('style', 'styleid', [vB_dB_Query::CONDITIONS_KEY => [
					'parentid' => -1,
					['field' => 'guid', vB_dB_Query::OPERATOR_KEY => vB_dB_Query::OPERATOR_ISNULL],
				]]);

				foreach ($styles AS $styleid)
				{
					$db->update('style',
						[
							'parentid' => $legacywritable['styleid'],
							'parentlist' => $styleid . ',' . $legacywritable['parentlist'],
						],
						['styleid' => $styleid]
					);

					//force the style lib to update the internal cache to avoid overwriting
					//the parentlist later on.
					$stylelib->fetchTemplateParentlist($styleid, true);
				}
			}
		}

		//we need to rebuild styles after this but the final upgrade script will handle that.
	}

	public function step_3()
	{
		$this->show_message(sprintf($this->phrase['vbphrase']['update_table_x'], TABLE_PREFIX . 'style', 2, 2));

		$db = vB::getDBAssertor();

		$id = $db->insert('style', [
			'title' => $this->phrase['install']['default_style'],
			'parentid' => -1,
			'parentlist' => '',
			'templatelist' => '',
			'replacements' => '',
			'userselect' => 0,
			'displayorder' => 1,
		]);


		//the parent list needs to refer to the styleid which we don't have until the record is created.
		if ($id)
		{
			$db->update('style', ['parentlist' => "$id,-1"], ['styleid' => $id]);
		}
	}
}

class vB_Upgrade_600a8 extends vB_Upgrade_Version
{
	//update the vB5 shim to
	public function step_1()
	{
		$this->show_message(sprintf($this->phrase['vbphrase']['update_table_x'], TABLE_PREFIX . 'style', 1, 2));

		$db = vB::getDBAssertor();

		//we could do this an one custom query but it's not worth the trouble.
		$legacy = $db->update('style',
			['displayorder' => 5],
			['guid' => 'vbulletin-theme-legacy-668e390263dc4ebd89f967b785bc47da']
		);
	}

	public function step_2()
	{
		$this->show_message(sprintf($this->phrase['vbphrase']['update_table_x'], TABLE_PREFIX . 'style', 2, 2));

		$db = vB::getDBAssertor();

		//we could do this an one custom query but it's not worth the trouble.
		$legacy = $db->update('style',
			['displayorder' => 10],
			['guid' => 'vbulletin-theme-parent-readonly-5660da3dd0cc42.92747689']
		);
	}
}

class vB_Upgrade_600b2 extends vB_Upgrade_Version
{
	public function step_1()
	{
		$this->set_option2('cpstyleimageext', 'default');
	}
}

class vB_Upgrade_601a3 extends vB_Upgrade_Version
{
	private $pmrouteguid = 'vbulletin-4ecbdacd6aac05.50909921';
	public function step_1()
	{
		$db = vB::getDBAssertor();
		$oldroute = $db->getRow('routenew', [
			'guid' => $this->pmrouteguid,
			'prefix' => 'privatemessage',
		]);
		if (!$oldroute)
		{
			$this->skip_message();
			return;
		}

		$conflict = $db->getRow('routenew', [vB_dB_Query::CONDITIONS_KEY => [
			['field' =>'prefix', 'value' => 'messagecenter', 'operator' => vB_dB_Query::OPERATOR_EQ],
			// ignore ones that are redirect301, which happens if there was a custom page with 'messagecenter' as the url, and
			// they moved the page to another URL.
			['field'=>'redirect301', 'operator' => vB_dB_Query::OPERATOR_ISNULL],
		]]);
		if ($conflict)
		{
			// we can't just dedupe this URL, because there are some places that hard-code the privatemessage/messagecenter URL,
			// particularly in JS
			$this->add_adminmessage('cannot_update_pm_route',
				[
					'dismissable' => 1,
					'status'  => 'undone',
				],
				false,
				[$this->LONG_VERSION]
			);
			$this->show_message(sprintf($this->phrase['version']['601a3']['cannot_update_pm_route'], $this->LONG_VERSION));
			return;
		}

		$this->show_message($this->phrase['version']['601a3']['updating_pm_route']);

		$prefix = vB5_Route_PrivateMessage::DEFAULT_PREFIX;
		$regex = $prefix . '/' . vB5_Route_PrivateMessage::REGEXP;
		$db->update('routenew',
			[
				'prefix' => $prefix,
				'regex' => $regex,
			],
			['guid' => $this->pmrouteguid]
		);

		// Clean up any redirect301's
		$conflict = $db->delete('routenew', [
			['field' =>'prefix', 'value' => 'messagecenter', 'operator' => vB_dB_Query::OPERATOR_EQ],
			['field'=>'redirect301', 'operator' => vB_dB_Query::OPERATOR_ISNOTNULL],
			// the default pm route SHOULDN'T have a redirect301 set to anything, but JUST IN CASE avoid nuking
			// the default route.
			['field' =>'guid', 'value' => $this->pmrouteguid, 'operator' => vB_dB_Query::OPERATOR_NE],
		]);
	}

	public function step_2()
	{
		$this->run_query(
			sprintf($this->phrase['vbphrase']['create_table'], TABLE_PREFIX . 'undolog'),
			"
				CREATE TABLE " . TABLE_PREFIX . "undolog (
					undoid INT NOT NULL AUTO_INCREMENT,
					class VARCHAR(50) NOT NULL,
					userid INT UNSIGNED NOT NULL,
					data BLOB NOT NULL,
					created INT NOT NULL,
					deleteafter INT NOT NULL,
					KEY lookup(userid, class),
					KEY deleteafter (deleteafter),
					PRIMARY KEY (undoid)
				) ENGINE = " . $this->hightrafficengine . "
			",
			self::MYSQL_ERROR_TABLE_EXISTS
		);
	}
}

class vB_Upgrade_601a4 extends vB_Upgrade_Version
{
	public function step_1()
	{
		$this->add_field2(
			'nodefield',
			'required',
			'TINYINT',
			[
				'attributes' => 'UNSIGNED',
				'null'       => false,
				'default'    => 0,
			]
		);
	}
}

class vB_Upgrade_602a1 extends vB_Upgrade_Version
{
	public function step_1()
	{
		$this->run_query(
			sprintf($this->phrase['vbphrase']['create_table'], TABLE_PREFIX . 'styleschedule'),
			"
				CREATE TABLE `" . TABLE_PREFIX . "styleschedule` (
					`scheduleid` SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
					`styleid` SMALLINT UNSIGNED NOT NULL,
					`enabled` TINYINT NOT NULL DEFAULT '0',
					`startdate` DATETIME NOT NULL DEFAULT '1000-01-01 00:00:00',
					`startdate_tzoffset` CHAR(4) NOT NULL DEFAULT '0',
					`enddate` DATETIME NOT NULL DEFAULT '9999-12-31 23:59:59',
					`enddate_tzoffset` CHAR(4) NOT NULL DEFAULT '0',
					`useyear` TINYINT NOT NULL DEFAULT '0',
					`priority` SMALLINT UNSIGNED NOT NULL DEFAULT '10',
					`overridechannelcustom` TINYINT NOT NULL DEFAULT '0',
					`overrideusercustom` TINYINT NOT NULL DEFAULT '0',
					`title` VARCHAR(250) NOT NULL DEFAULT ''
				) ENGINE = " . $this->hightrafficengine . "
			",
			self::MYSQL_ERROR_TABLE_EXISTS
		);
	}

	public function step_2()
	{
		// This used to insert sample schedules, but due to the issues with getting this working on fresh installs,
		// we decided to scrap the samples for now.
		$this->skip_message();
	}
}

class vB_Upgrade_602a2 extends vB_Upgrade_Version
{
	// only affects alpha 1 builds only, `timezoneoffset` was spit into `startdate_tzoffset` &
	// `enddate_tzoffset`.
	public function step_1()
	{
		if ($this->field_exists('styleschedule', 'startdate_tzoffset'))
		{
			$this->skip_message();
			return;
		}

		// AFAIK the actual field changes are done at the end, so we can't do things like
		// "add column and copy data" in a single step.
		$this->add_field2(
			'styleschedule',
			'startdate_tzoffset',
			'CHAR',
			[
				'length' => 4,
				'null'       => false,
				'default'    => '0',
			]
		);
		$this->add_field2(
			'styleschedule',
			'enddate_tzoffset',
			'CHAR',
			[
				'length' => 4,
				'null'       => false,
				'default'    => '0',
			]
		);
	}

	// copy alpha1's timezoneoffset to alpha2's enddate_tzoffset
	public function step_2()
	{
		if (!$this->field_exists('styleschedule', 'timezoneoffset'))
		{
			$this->skip_message();
			return;
		}

		$this->show_message($this->phrase['version']['602a2']['adding_start_end_timezones']);
		$assertor = vB::getDbAssertor();
		$assertor->assertQuery('vBInstall:copyStyleschedleTzOffsetToStartEndTzoffsets');
		$this->drop_field2('styleschedule', 'timezoneoffset');
	}

	// Some of the reaction save logic now requires this new table to exist first.
	public function step_3()
	{
		$this->run_query(
			sprintf($this->phrase['vbphrase']['create_table'], TABLE_PREFIX . 'reactionoption'),
			"
				CREATE TABLE `" . TABLE_PREFIX . "reactionoption` (
					`votetypeid` SMALLINT UNSIGNED NOT NULL,
					`enabled` TINYINT SIGNED NOT NULL DEFAULT '0',
					`user_rep_factor` TINYINT SIGNED NOT NULL DEFAULT '0',
					`user_like_countable` TINYINT SIGNED NOT NULL DEFAULT '0',
					`emojihtml` MEDIUMTEXT NOT NULL,
					`system` TINYINT SIGNED NOT NULL DEFAULT '0',
					`guid` VARCHAR(255) NULL DEFAULT NULL,
					`order` INT SIGNED NOT NULL DEFAULT '0',
					`filedataid` INT UNSIGNED NOT NULL DEFAULT '0',
					UNIQUE INDEX `votetypeid` (`votetypeid`),
					INDEX `filedataid` (`filedataid`)
				) ENGINE = " . $this->hightrafficengine . "
			",
			self::MYSQL_ERROR_TABLE_EXISTS
		);
	}

	// Add the new reactions for VB6-213
	public function step_4()
	{
		// Re-using old version's upgrade phrase.
		$this->show_message($this->phrase['version']['600a1']['inserting_topic_reactions']);
		// Updated addDefaultReactions() will add AND enable the new emojis by default.
		$this->addDefaultReactions();
	}
}

class vB_Upgrade_602a3 extends vB_Upgrade_Version
{
	public function step_1($data = [])
	{
		if (!$this->tableExists('announcement'))
		{
			$this->skip_message();
			return;
		}

		if (empty($data['startat']))
		{
			$this->show_message(sprintf($this->phrase['vbphrase']['update_table_x'], TABLE_PREFIX . 'notice', 1, 1));
		}

		//We need to process these one by one but there shouldn't be that many.  Batch just to be sure.
		$walker = vB_UpdateTableWalker::getSimpleTableWalker(
			vB::getDBAssertor(),
			$this->getBatchSize(20, __FUNCTION__),
			'vBInstall:announcement',
			'announcementid',
		);

		$walker->setCallback(function($startat, $nextid)
		{
			$db = vB::getDBAssertor();
			$notice = vB_Library::instance('notice');

			$announcements = $db->select('vBInstall:announcement', [
				['field' => 'announcementid', 'value' => $startat, 'operator' => vB_dB_Query::OPERATOR_GTE],
				['field' => 'announcementid', 'value' => $nextid, 'operator' => vB_dB_Query::OPERATOR_LT],
			]);

			$row = $db->getRow('vBForum:notice', [
				vB_dB_Query::TYPE_KEY => vB_dB_Query::QUERY_SUMMARY,
				vB_dB_Query::COLUMNS_KEY => ['MAX(displayorder)'],
			]);
			$displayorder = $row['max'];

			foreach ($announcements AS $announcement)
			{
				//the notice and announcements use the same bitfield values except that notices don't
				//have signatures.  However we need to expand to an array if we plan to use the save
				//function.  Hard code incase we want to rename the bitfields -- it will mess with
				//reentrency if we rely on the bitfield array.
				$announcementoptions = $announcement['announcementoptions'];
				$noticeoptions = [
					'allowbbcode' => boolval($announcementoptions & 1),
					'allowhtml' => boolval($announcementoptions & 2),
					'allowsmilies' => boolval($announcementoptions & 4),
					'parseurl' => boolval($announcementoptions & 8),
				];

				//Note that unixtimestamps are in UTC regardless of what TZ you tell it in the constructor, but we happen to want that.
				//The timestamps for announcements appear to be for UTC though that's not explict
				$start = new DateTimeImmutable('@' . $announcement['startdate']);
				$end = new DateTimeImmutable('@' . $announcement['enddate']);
				$format = 'd-m-Y';

				$displayorder += 10;
				$criteria = [
					'is_date_range' => ['condition1' => $start->format($format), 'condition2' => $end->format($format), 'condition3' => 1],
				];

				if ($announcement['nodeid'] != -1)
				{
					$criteria['browsing_forum_x_and_children'] = ['condition1' => $announcement['nodeid']];
				}

				// Notice titles may not be empty. Just fill it with something.
				if (empty($announcement['title']))
				{
					$announcement['title']  = 'Untitled';
				}

				$notice->save([
					'title' => $announcement['title'],
					'text' => $announcement['pagetext'],
					'displayorder' => $displayorder,
					'active' => 1,
					'persistent' => 1,
					'dismissible' => 1,
					'noticeoptions' => $noticeoptions,
					'criteria' => $criteria,
				]);
			}
		});

		return $this->updateByWalker($walker, $data);
	}

	public function step_2()
	{
		$this->show_message(sprintf($this->phrase['vbphrase']['update_table_x'], TABLE_PREFIX . 'widgetinstance', 1, 1));

		$db = vB::getDBAssertor();
		//there should only be one result.
		$widgetid = $db->getColumn('widget', 'widgetid', ['guid' => 'vbulletin-widget_announcement-4eb423cfd6dea7.34930845']);
		$widgetid = current($widgetid);

		$widgetinstances = $db->getColumn('widgetinstance', 'widgetinstanceid', ['widgetid' => $widgetid]);
		vB_Library::instance('widget')->deleteWidgetInstances($widgetinstances, true);
	}

	public function step_3()
	{
		$this->show_message(sprintf($this->phrase['core']['delete_widget_x'], 'Announcement'));

		//thre aren't a ton of widgets/widget instances in a database.  Shouldn't need to batch and doing so will almost certainly
		//be less efficient/make things take longer
		$db = vB::getDBAssertor();

		//this is somewhat general but need to recursively clean up after container widgets -- which the announcement widget is not
		$db->assertQuery('vBInstall:deleteWidget', ['guid' => 'vbulletin-widget_announcement-4eb423cfd6dea7.34930845']);
	}

	public function step_4()
	{
		$this->show_message(sprintf($this->phrase['vbphrase']['update_table_x'], TABLE_PREFIX . 'hook', 1, 1));
		$db = vB::getDBAssertor();
		$db->delete('hook', ['hookname' => ['announcement_total', 'announcement_after_list', 'announcement_no_announcement']]);
	}

	public function step_5()
	{
		$this->drop_field2('rssfeed', 'endannouncement');
	}

	public function step_6()
	{
		$this->drop_table('announcement');
	}

	public function step_7()
	{
		$this->drop_table('announcementread');
	}

	public function step_8()
	{
		$this->show_message(sprintf($this->phrase['vbphrase']['update_table_x'], TABLE_PREFIX . 'contenttype', 1, 1));
		$announcementtypeid = vB_Types::instance()->getContentTypeID('vBForum_Announcement');
		if ($announcementtypeid)
		{
			$db = vB::getDBAssertor();
			$db->delete('vBForum:contenttype', ['contenttypeid' => $announcementtypeid]);
		}
	}
}

class vB_Upgrade_603a1 extends vB_Upgrade_Version
{
	public function step_1()
	{
		$extension = 'mov';
		$db = vB::getDbAssertor();
		$row = $db->getRow('vBForum:attachmenttype', ['extension' => $extension]);

		if (!$row)
		{
			$query = getAttachmenttypeInsertQuery($this->db, [$extension]);
			$this->run_query(
				sprintf($this->phrase['core']['altering_x_table'], 'attachmenttype', 1, 1),
				$query
			);
		}
		else
		{
			$this->skip_message();
		}
	}
}

class vB_Upgrade_603a3 extends vB_Upgrade_Version
{
	public function step_1()
	{
		$this->run_query(
			sprintf($this->phrase['vbphrase']['create_table'], TABLE_PREFIX . 'reactionoption'),
			"
				CREATE TABLE `" . TABLE_PREFIX . "reactionoption` (
					`votetypeid` SMALLINT UNSIGNED NOT NULL,
					`enabled` TINYINT SIGNED NOT NULL DEFAULT '0',
					`user_rep_factor` TINYINT SIGNED NOT NULL DEFAULT '0',
					`user_like_countable` TINYINT SIGNED NOT NULL DEFAULT '0',
					`emojihtml` MEDIUMTEXT NOT NULL,
					`system` TINYINT SIGNED NOT NULL DEFAULT '0',
					`guid` VARCHAR(255) NULL DEFAULT NULL,
					`order` INT SIGNED NOT NULL DEFAULT '0',
					`filedataid` INT UNSIGNED NOT NULL DEFAULT '0',
					UNIQUE INDEX `votetypeid` (`votetypeid`),
					INDEX `filedataid` (`filedataid`)
				) ENGINE = " . $this->hightrafficengine . "
			",
			self::MYSQL_ERROR_TABLE_EXISTS
		);
	}

	// Populate `reactionoption` with default values. Also pulls the "enabled" data that was, before this version, ONLY saved
	// in the datastore.
	public function step_2()
	{
		// TODO: add step to delete legacy data after we're sure we don't need it?

		$assertor = vB::getDbAssertor();
		$check = $assertor->getRow('reactionoption');
		if (!empty($check))
		{
			$this->skip_message();
			return;
		}
		else
		{
			$this->show_message(sprintf($this->phrase['vbphrase']['update_table'], TABLE_PREFIX . 'reactionoption'));
		}


		/** @var vB_Library_Reactions */
		$lib = vB_Library::instance('reactions');
		// this function should not cross wires with `reactions` table so should be safe to use.
		$knownreactions = $lib->getReactionsNodevotetypes();
		$reactionLabelsByKey = array_column($knownreactions, 'label', 'votetypeid');
		$reactionIdsByLabel =  array_column($knownreactions, 'votetypeid', 'label');

		// Depending on the specific upgrade path (e.g. 5 -> 602, 600 -> 602 etc) some might already
		// have the option record set up. Only handle the ones that are missing, expected to be in the
		// 600 or 601 upgrade to 602.
		$alreadyExists = $assertor->getRows('reactionoption', [], false, 'votetypeid');
		$enabled = $this->getLegacyEmojisEnabledStatus();
		$enabledTypeids = [];
		foreach ($enabled AS $__label => $__enabled)
		{
			$__id = $reactionIdsByLabel[$__label];
			if (!isset($alreadyExists[$__id]))
			{
				$enabledTypeids[] = $__id;
			}
		}

		if (!$enabledTypeids)
		{
			$this->skip_message();
			return;
		}


		// We don't want this dynamic, because we're trying to set up the default value.
		// If any new reactions are added after this step, we don't care -- their rep/count values
		// should be handled by the new adminCP pages, or by the upgrade steps adding them if
		// they should start off with some defaults.
		$reputables = [
			'grinning face' => 1,
			'smiling face with hearts' => 1,
			'smiling face with sunglasses' => 1,
			'thumbs up' => 1,
			'face with tears of joy' => 1,
		];

		$insert = [];
		foreach ($enabledTypeids AS $__votetypeid)
		{
			$__label = $reactionLabelsByKey[$__votetypeid];
			// If we do NOT have the enabled DS set, assume this is the first time & enable all by default.
			// Otherwise, if it's missing in the enabled list, keep it disabled.
			// Note taht the legacy datstore item was keyed by label instead of votetypeid.
			$__enabled = $enabled ? ($enabled[$__label] ?? 0) : 1;
			// At this time, thke reputables & countables were hard-coded & set to be the same as each other. Avoiding using the
			// getReputableTypesAndFactors() ETC to avoid weird cyclic dependencies between reactionoption table & the library
			// during this step that's supposed to populate the table for the first time.
			$__rep = $reputables[$__label] ?? 0;
			$__count = $reputables[$__label] ?? 0;
			$insert[] = [
				$__votetypeid,
				$__enabled,
				$__rep,
				$__count,
			];
		}
		// Insert several prefixsets and prefixes
		$assertor->insertMultiple(
			'reactionoption',
			['votetypeid', 'enabled', 'user_rep_factor', 'user_like_countable'],
			$insert
		);
	}

	// Trying to remove some circular dependencies -- the upgrade step is supposed to populate `reactionoption` for the first
	// time, but vB_Library_Reactions::getEmojisEnabledStatus() may end up hitting the same table and rebuild data from that.
	private function getLegacyEmojisEnabledStatus()
	{
		$datastore = vB::getDatastore();
		$enabled = $datastore->getValue(vB_Library_Reactions::EMOJI_ENABLED_LEGACY_DS_KEY);

		if ($enabled)
		{
			$enabled = json_decode($enabled, true);
		}
		// handle cases wehre json decode fails, or ds::get missed
		$enabled = ($enabled ? $enabled : []);

		return $enabled;
	}

	public function step_3()
	{
		// Add index for recalculating user.totallikes
		$this->add_index2('nodevote', 'totallikes_agg_helper', ['userid', 'votetypeid', 'whovoted',]);
	}

	public function step_4()
	{
		// AFAIK the actual field changes are done at the end, so we can't do things like
		// "add column and copy data" in a single step.
		$this->add_field2(
			'reactionoption',
			'emojihtml',
			'MEDIUMTEXT',
			[
				'null'       => true,
				'default'    => null,
			]
		);
		$this->add_field2(
			'reactionoption',
			'system',
			'TINYINT',
			[
				'attributes' => 'SIGNED',
				'null'       => false,
				'default'    => '0',
			]
		);
		$this->add_field2(
			'reactionoption',
			'guid',
			'VARCHAR',
			[
				'length' => 255,
				'null'       => true,
				'default'    => null,
			]
		);
		$this->add_field2(
			'reactionoption',
			'order',
			'INT',
			[
				'attributes' => 'SIGNED',
				'null'       => false,
				'default'    => '0',
			]
		);
	}

	// fill data for the system default reactions
	public function step_5()
	{
		// Check for re-entry. If the admin uses this after the new 603 features and re-names a default system reaction,
		// we might misidentify certain ones that used to be keyed by label only and cause problems below. And there's
		// probably no reason to run the updates again, since doing so would just possibly wipe customized emojihtml or
		// order data.
		$assertor = vB::getDbAssertor();
		$check = $assertor->getRow('reactionoption', ['system' => 1]);
		if (!empty($check))
		{
			$this->skip_message();
			return;
		}

		$this->show_message(sprintf($this->phrase['vbphrase']['update_table'], TABLE_PREFIX . 'reactionoption'));
		// Side note, using this function instead of hard-coding the *current* list means
		// future updates to the list would affect this step.
		$defaultLabels = $this->fetchDefaultReactionLabels();
		// Defaults at the time of this writing:
		// $defaults = [
		// 	'grinning face',
		// 	'smiling face with hearts',
		// 	'smiling face with sunglasses',
		// 	'enraged face',
		// 	'nauseated face',
		// 	'thumbs up',
		// 	'thumbs down',
		// 	// Added in VB6-213
		// 	'face with tears of joy',
		// 	'face blowing a kiss',
		// 	'disappointed face',
		// 	'hot beverage',
		// ];

		/** @var vB_Library_Reactions */
		$lib = vB_Library::instance('reactions');
		$knownreactions = $lib->getReactionsNodevotetypes();
		$reactionIdsByLabel =  array_column($knownreactions, 'votetypeid', 'label');
		['fulldata' => $emojionlydata,] = $lib->loadSourceEmojisData();
		$byLabel = array_column($emojionlydata, null, 'label');
		// All existing default reactions should have `reactionoption` records by this point.
		// Running on that assumption.
		foreach ($defaultLabels AS $__label)
		{
			$__data = $byLabel[$__label] ?? [];
			$__id = $reactionIdsByLabel[$__label];
			if (empty($__data))
			{
				continue;
			}
			$assertor->update('reactionoption',
				[
					'system' => 1,
					'guid' => $__label,
					'emojihtml' => $__data['emojihtml'],
					'order' => $__data['order'],
				],
				['votetypeid' => $__id]
			);
		}
		$lib->purgeReactionCaches();
	}

	public function step_6()
	{
		// delete old no longer used datastore entries...
		$datastore = vB::getDatastore();
		$delete = [
			'vb_reactions_userreputables',
			'vb_reactions_usercountables',
			'vb_emoji_enabled_byid',
		];
		foreach ($delete AS $__deleteme)
		{
			$this->show_message(sprintf($this->phrase['core']['remove_datastore_x'], $__deleteme));
			$datastore->delete($__deleteme);
		}
	}

	public function step_7()
	{
		$this->add_field2(
			'reactionoption',
			'filedataid',
			'INT',
			[
				'attributes' => 'UNSIGNED',
				'null'       => false,
				'default'    => 0,
			]
		);
		/** @var vB_Library_Reactions */
		$lib = vB_Library::instance('reactions');
		$lib->purgeReactionCaches();
	}

	public function step_8()
	{
		$this->add_index2('reactionoption', 'filedataid', ['filedataid',]);
	}

	public function step_9()
	{
		$this->add_field2(
			'channel',
			'topicexpiretype',
			'ENUM',
			[
				'attributes' => "('none', 'soft', 'hard')",
				'null'       => false,
				'default'    => 'none',
			]
		);
		$this->add_field2(
			'channel',
			'topicexpireseconds',
			'INT',
			[
				'attributes' => 'UNSIGNED',
				'null'       => false,
				'default'    => 0,
			]
		);
	}

	public function step_10()
	{
		$this->add_index2('channel', 'topicexpirehelper', ['topicexpiretype', 'nodeid', 'topicexpireseconds']);
	}
}

class vB_Upgrade_603a4 extends vB_Upgrade_Version
{
	public function step_1()
	{
		$db = vB::getDBAssertor();
		$settingnames = [
			'google_ownership_verification_tag',
			'google_ownership_verification_enable',
			'bing_ownership_verification_tag',
			'bing_ownership_verification_enable',
		];

		$settings = $db->getColumn('setting', 'value', ['varname' => $settingnames], false, 'varname');
		// If we don't have all of the settings here (either because we're upgrading before they existed
		// or because we're reentering after they were renamed/removed) then the is nothing useful we can do.
		if (array_diff($settingnames, array_keys($settings)))
		{
			$this->skip_message();
			return;
		}

		// only retain the values if the setting is enabled.
		$newvalue = [];
		if ($settings['google_ownership_verification_enable'])
		{
			$newvalue[] = $settings['google_ownership_verification_tag'];
		}

		if ($settings['bing_ownership_verification_enable'])
		{
			$newvalue[] = $settings['bing_ownership_verification_tag'];
		}

		// We'll use the "google" value as the one that continues to exist.  We'll rename it later.
		// The "enable" options will be removed but as a standard setting update.
		$this->set_option2('google_ownership_verification_tag', implode("\n\n", $newvalue), 1, 2);
	}

	public function step_2()
	{
		$db = vB::getDBAssertor();
		$settingnames = [
			'ga_enabled',
			'ga_code',
		];

		$settings = $db->getColumn('setting', 'value', ['varname' => $settingnames], false, 'varname');

		// we only really need ga_enabled but check that ga_code also exists so we don't try to update
		// an option that isn't there.  If it's enabled there is nothing to do.
		if (array_diff($settingnames, array_keys($settings)) OR $settings['ga_enabled'])
		{
			$this->skip_message();
			return;
		}

		//otherwise blank the value (we'll be removing the enabled setting later).
		$this->set_option2('ga_code', '', 2, 2);
	}

	public function step_3()
	{
		$this->rename_option('ga_code', 'footer_code', 1, 2);
	}

	public function step_4()
	{
		$this->rename_option('google_ownership_verification_tag', 'header_code', 2, 2);
	}
}

class vB_Upgrade_603b1 extends vB_Upgrade_Version
{
	public function step_1($data = [])
	{
		if (empty($data['startat']))
		{
			$this->show_message(sprintf($this->phrase['vbphrase']['update_table_x'], TABLE_PREFIX . 'user', 1, 1));
		}

		$walker = vB_UpdateTableWalker::getSimpleTableWalkerFilter(
			vB::getDBAssertor(),
			$this->getBatchSize('xlarge', __FUNCTION__),
			'user',
			'userid',
			['privacyconsent' => -1]
		);
		$walker->setCallbackQuery('vBInstall:clearUserDelete');
		return $this->updateByWalker($walker, $data);
	}
}

/*======================================================================*\
|| ####################################################################
|| # Downloaded: 16:45, Wed Feb 28th 2024
|| # CVS: $RCSfile$ - $Revision$
|| ####################################################################
\*======================================================================*/
