<?php
/*========================================================================*\
|| ###################################################################### ||
|| # vBulletin 6.0.3 - Licence Number LN98610432
|| # ------------------------------------------------------------------ # ||
|| # Copyright 2000-2024 MH Sub I, LLC dba vBulletin. All Rights Reserved.  # ||
|| # This file may not be redistributed in whole or significant part.   # ||
|| # ----------------- VBULLETIN IS NOT FREE SOFTWARE ----------------- # ||
|| # http://www.vbulletin.com | http://www.vbulletin.com/license.html   # ||
|| ###################################################################### ||
\*========================================================================*/

class vB5_Frontend_Controller_Dashboard extends vB5_Frontend_Controller
{
	protected $api;
	function __construct()
	{
		parent::__construct();
		$this->api = Api_InterfaceAbstract::instance();
	}

	public function actionUndo()
	{
		$currentuser = vB5_User::instance();
		if (empty($currentuser['userid']))
		{
			$this->showErrorPage(['no_permission_logged_out']);
			return;
		}

		$router = vB5_ApplicationAbstract::instance()->getRouter();
		$queryParams = $router->getQueryParameters();

		if (empty($queryParams['undoid']))
		{
			$this->showErrorPage(['invalid_data']);
			return;
		}
		$callNamed = true;
		$vars = [
			'undoid' => intval($queryParams['undoid']),
		];
		$result = $this->api->callApi('notification', 'undoUnsubscribe', $vars, $callNamed);

		if (!empty($result['errors']))
		{
			return $this->showErrorPage($result['errors']);
		}

		$phrasesToFetch = [
			'change_notification_settings',
			'resubbed_to_following_nodes',
			'unsubscribe_pagetitle',
			'view_your_subs',
			//'changing_password_but_currently_logged_in_msg',
		];
		$phrases = $this->api->callApi('phrase', 'fetch', [$phrasesToFetch]);

		$message = $phrases['resubbed_to_following_nodes'];
		foreach ($result['subscribed'] AS ['nodeid' => $__id, 'title' => $__title])
		{
			$url = vB5_Route::buildUrl('node|fullurl', ['nodeid' => $__id]);
			$message .= "<br/>\n<a href=\"" . htmlentities($url) . "\">{$__title}</a>" ;
		}

		$profilelink =  vB5_Route::buildUrl('subscription|fullurl', ['tab' => 'subscriptions', 'userid' => $currentuser['userid']]);
		$message .= "<br/><br/>\n" . vsprintf($phrases['view_your_subs'], [$profilelink]);

		$settingslink = vB5_Route::buildUrl('settings|fullurl', ['tab' => 'notifications']);
		$message .= "<br/><br/>\n" . vsprintf($phrases['change_notification_settings'], [$settingslink]);

		$this->showMessagePage($phrases['unsubscribe_pagetitle'], $message);
	}

	public function actionUnsubscribe()
	{
		$currentuser = vB5_User::instance();
		if (empty($currentuser['userid']))
		{
			$this->showErrorPage(['no_permission_logged_out']);
			return;
		}

		$router = vB5_ApplicationAbstract::instance()->getRouter();
		$queryParams = $router->getQueryParameters();

		if (empty($queryParams['sentbynodeid']))
		{
			$this->showErrorPage(['invalid_data']);
			return;
		}
		$callNamed = true;
		$vars = [
			'sentbynodeid' => intval($queryParams['sentbynodeid']),
		];
		$result = $this->api->callApi('notification', 'unsubscribeFromNotification', $vars, $callNamed);

		if (!empty($result['errors']))
		{
			return $this->showErrorPage($result['errors']);
		}

		$phrasesToFetch = [
			'already_unsubscribed',
			'change_notification_settings',
			'to_undo_click',
			'unsubbed_from_following_nodes',
			'unsubscribe_pagetitle',
			'view_your_subs',
		];
		$phrases = $this->api->callApi('phrase', 'fetch', [$phrasesToFetch]);


		if (empty($result['unsubscribed']))
		{
			$message = $phrases['already_unsubscribed'];
		}
		else
		{
			$message = $phrases['unsubbed_from_following_nodes'];
			foreach ($result['unsubscribed'] AS ['nodeid' => $__id, 'title' => $__title])
			{
				$url = vB5_Route::buildUrl('node|fullurl', ['nodeid' => $__id]);
				$message .= "<br/>\n<a href=\"" . htmlentities($url) . "\">{$__title}</a>" ;
			}

			if (!empty($result['undoid']))
			{
				$url = vB5_Route::buildUrl('dashboard|fullurl', ['action' => 'undo'], ['undoid' => $result['undoid']]);
				$message .= "<br/><br/>\n" . vsprintf($phrases['to_undo_click'], [$url]);
			}
		}

		$profilelink =  vB5_Route::buildUrl('subscription|fullurl', ['tab' => 'subscriptions', 'userid' => $currentuser['userid']]);
		$message .= "<br/><br/>\n" . vsprintf($phrases['view_your_subs'], [$profilelink]);

		$settingslink = vB5_Route::buildUrl('settings|fullurl', ['tab' => 'notifications']);
		$message .= "<br/><br/>\n" . vsprintf($phrases['change_notification_settings'], [$settingslink]);

		$this->showMessagePage($phrases['unsubscribe_pagetitle'], $message);
	}

	public function index()
	{
		$this->showErrorPage(['invalid_data']);
	}
}

/*=========================================================================*\
|| #######################################################################
|| # Downloaded: 16:45, Wed Feb 28th 2024
|| # CVS: $RCSfile$ - $Revision: 111676 $
|| #######################################################################
\*=========================================================================*/
